# copyright 2018 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as publishged by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref tests for ARK identifier generation."""

from unittest import TestCase

import psycopg2
import psycopg2.errorcodes

from cubicweb.devtools import (
    PostgresApptestConfiguration,
    testlib,
)

from cubicweb_saem_ref.ark import (
    ARK_RGX,
    insert_ark,
    match as ark_match,
)
import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class ArkRgxTC(TestCase):

    def test_syntax(self):
        """Make string interpolation is correct."""
        self.assertEqual(
            ARK_RGX.pattern,
            r'^(ark:/)?(?P<naan>\d+)/(?P<name>rf\w{7}g)(/(?P<qualifier>.{10}))?$',
        )

    def test(self):
        match = ark_match('ark:/12345/rf%sg' % ('q' * 7))
        self.assertTrue(match)
        self.assertEqual(match.group('naan'), '12345')
        self.assertEqual(match.group('name'), 'rfqqqqqqqg')

    def test_ext(self):
        match = ark_match('ark:/12345/name', external=True)
        self.assertTrue(match)
        self.assertEqual(match.group('naan'), '12345')
        self.assertEqual(match.group('name'), 'name')

    def test_no_scheme(self):
        match = ark_match('12345/rf%sg' % ('q' * 7))
        self.assertTrue(match)
        self.assertEqual(match.group('naan'), '12345')
        self.assertEqual(match.group('name'), 'rfqqqqqqqg')

    def test_ext_no_scheme(self):
        match = ark_match('12345/name', external=True)
        self.assertTrue(match)
        self.assertEqual(match.group('naan'), '12345')
        self.assertEqual(match.group('name'), 'name')

    def test_qualified(self):
        ark = '0/rfr8hps6ng/h4050n4j02'
        match = ark_match(ark)
        self.assertTrue(match)
        self.assertEqual(match.group('naan'), '0')
        self.assertEqual(match.group('name'), 'rfr8hps6ng')
        self.assertEqual(match.group('qualifier'), 'h4050n4j02')

    def test_ext_qualified(self):
        ark = '666/name/qualifier'
        match = ark_match(ark, external=True)
        self.assertTrue(match)
        self.assertEqual(match.group('naan'), '666')
        self.assertEqual(match.group('name'), 'name')
        self.assertEqual(match.group('qualifier'), 'qualifier')

    def test_tesutils_match_ark_qualified(self):
        ark = '0/rfr8hps6ng/h4050n4j02'

        def check(match):
            self.assertTrue(match)
            self.assertEqual(match.group(1), '/h4050n4j02')

        check(testutils.match_ark(ark))
        check(testutils.match_ark_uri('ark:/' + ark))


class ArkServiceTC(testlib.CubicWebTC):
    # This could be a plain unittest.TestCase as we don't use CubicWeb's
    # database. But on the other hand, CubicWebTC makes it easier to get a
    # postgres configuration handling setup/teardown.

    configcls = PostgresApptestConfiguration

    def connect(self):
        return self.repo.system_source.get_connection()

    def test_unique_generated(self):
        """We should be able to produce 20 ARK identifiers of length 4 with a
        2-characters prefix and 1 control character.
        """
        n = 20
        arks = []
        with self.connect() as conn:
            with conn.cursor() as cu:
                for _ in range(n):
                    cu.execute("SELECT * from gen_ark(%s, %s, %s, %s, %s);",
                               (0, 4, 'rf', 'x', 10))
                    arks.append(cu.fetchone()[0])
                    conn.commit()
                cu.execute("SELECT * from ark")
                r = cu.fetchall()
        self.assertEqual(len(set(r)), n, len(r))
        self.assertCountEqual([ark for _, ark, _ in r], arks)

    def test_gen_ark_maxit(self):
        with self.connect() as conn:
            with conn.cursor() as cu:
                with self.assertRaises(psycopg2.InternalError) as cm:
                    cu.execute("SELECT * from gen_ark(%s, %s, %s, %s, %s);",
                               (0, 4, 'rf', 'x', 0))
                exc = cm.exception
                self.assertEqual(psycopg2.errorcodes.lookup(exc.pgcode),
                                 'ASSERT_FAILURE')
                self.assertIn('maximum number of iterations: 0',
                              str(exc))

    def test_gen_qualified_ark_maxit(self):
        with self.connect() as conn:
            with conn.cursor() as cu:
                cu.execute("INSERT INTO ark VALUES (%s, %s, DEFAULT);",
                           (0, "parent", ))
                conn.commit()
                with self.assertRaises(psycopg2.InternalError) as cm:
                    cu.execute("SELECT * from gen_qualified_ark(%s, %s, %s, %s);",
                               (0, "parent", 7, 0))
                exc = cm.exception
                self.assertEqual(psycopg2.errorcodes.lookup(exc.pgcode),
                                 'ASSERT_FAILURE')
                self.assertIn('maximum number of iterations: 0',
                              str(exc))

    def test_qualifier(self):
        def insert(qualifier=None):
            with self.connect() as conn:
                with conn.cursor() as cu:
                    cu.execute("INSERT INTO ark VALUES (%s, %s, %s);",
                               (0, "he", qualifier))
        insert("ah")
        with self.assertRaises(psycopg2.IntegrityError) as cm:
            insert("ah")
        self.assertIn(
            'duplicate key value violates unique constraint',
            str(cm.exception),
        )
        insert("he")

    def test_dup(self):
        def insert():
            with self.connect() as conn:
                with conn.cursor() as cu:
                    cu.execute("INSERT INTO ark VALUES (42, 'he', 'aha');")
        insert()
        with self.assertRaises(psycopg2.IntegrityError) as cm:
            insert()
        self.assertIn(
            'duplicate key value violates unique constraint',
            str(cm.exception),
        )

    def test_generate_qualifier_no_name_existing(self):
        with self.connect() as conn:
            with conn.cursor() as cu:
                with self.assertRaises(psycopg2.DataError) as cm:
                    cu.execute("SELECT * FROM gen_qualified_ark(%s, %s, %s, 20);",
                               (1, "doesnotexist", 3))
        self.assertIn('no ark record matching "1/doesnotexist" found',
                      str(cm.exception))

    def test_generate_qualifier_name_existing(self):
        with self.connect() as conn:
            with conn.cursor() as cu:
                cu.execute("INSERT INTO ark VALUES (%s, %s, DEFAULT);",
                           (12345, "exists", ))
                cu.execute("SELECT * FROM gen_qualified_ark(%s, %s, %s, 20);",
                           (12345, "exists", 3))
                cu.execute("SELECT * FROM ark"
                           " WHERE ark.naan = 12345 AND ark.name = 'exists'"
                           " AND NOT ark.qualifier = '';")
                (_, _, qualifier), = cu.fetchall()
        self.assertEqual(len(qualifier), 3, qualifier)

    def test_insert_ark(self):
        with self.admin_access.cnx() as cnx:
            ark = insert_ark(cnx, '12345', u'bob')
            cnx.commit()
            self.assertEqual(ark, u'12345/bob')
            results = cnx.system_sql('SELECT naan, name, qualifier FROM ark;').fetchall()
            self.assertEqual(results, [(12345, u'bob', u'')])

    def test_insert_ark_qualifier(self):
        with self.admin_access.cnx() as cnx:
            ark = insert_ark(cnx, '12345', u'bob', u'1')
            cnx.commit()
            self.assertEqual(ark, u'12345/bob/1')
            results = cnx.system_sql('SELECT naan, name, qualifier FROM ark;').fetchall()
            self.assertEqual(results, [(12345, u'bob', u'1')])


if __name__ == '__main__':
    import unittest
    unittest.main()



# coding: utf-8
# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.devtools import PostgresApptestConfiguration

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class SKOSImportTC(CubicWebTC):
    configcls = PostgresApptestConfiguration

    def test_datafeed_source(self):
        with self.admin_access.repo_cnx() as cnx:
            url = u'file://%s' % self.datapath('skos.rdf')
            cnx.create_entity('CWSource', name=u'mythesaurus', type=u'datafeed', parser=u'rdf.skos',
                              url=url)
            # create some agent and related objects
            agent = testutils.authority_record(cnx, u'bob')
            function = cnx.create_entity('AgentFunction', name=u'sponge', function_agent=agent)
            cnx.commit()
            # create some external uri and link it to place, function and information entities
            cnx.create_entity('ExternalUri',
                              cwuri=u'http://data.culture.fr/thesaurus/resource/ark:/67717/T1-543',
                              uri=u'http://data.culture.fr/thesaurus/resource/ark:/67717/T1-543',
                              reverse_equivalent_concept=function)
            cnx.commit()
            self.function_eid = function.eid
        dfsource = self.repo.sources_by_uri['mythesaurus']
        # test creation upon initial pull
        with self.repo.internal_cnx() as cnx:
            dfsource.pull_data(cnx, force=True, raise_on_error=True)
        with self.admin_access.client_cnx() as cnx:
            scheme = cnx.find(
                'ConceptScheme',
                cwuri='http://data.culture.fr/thesaurus/resource/ark:/67717/Matiere'
            ).one()
            self.assertEqual(scheme.ark, u'67717/Matiere')
            self.assertEqual(set(c.ark for c in scheme.top_concepts),
                             set(['67717/T1-503', '67717/T1-543']))
            self.assertEqual(scheme.cw_adapt_to('IWorkflowable').state, 'draft')
            # ensure the external uri has been replaced by the concept and deleted
            concept = cnx.find('Concept', ark='67717/T1-543').one()
            function = cnx.entity_from_eid(function.eid)
            function.cw_clear_all_caches()
            self.assertEqual(function.equivalent_concept[0].eid, concept.eid)
            # XXX also unique extid  pb when skos import will support externaluri

    def test_skos_source_no_ark(self):
        with self.admin_access.repo_cnx() as cnx:
            cnx.create_entity('SKOSSource', name=u'mythesaurus', ark_naa=testutils.naa(cnx),
                              url=u'file://%s' % self.datapath('skos_no_ark.rdf'))
            cnx.commit()
        dfsource = self.repo.sources_by_uri['mythesaurus']
        # test creation upon initial pull
        with self.repo.internal_cnx() as cnx:
            dfsource.pull_data(cnx, force=True, raise_on_error=True)
        with self.admin_access.client_cnx() as cnx:
            scheme = cnx.find('ConceptScheme').one()
            scheme_ark = scheme.ark
            self.assertTrue(scheme_ark.startswith('0/rf'))
            concepts = list(scheme.top_concepts)
            self.assertEqual(len(concepts), 2)
            concept_arks = [c.ark for c in concepts]
            self.assertTrue(all(ark.startswith(scheme_ark + '/') for ark in concept_arks),
                            concept_arks)


class LCSVImportTC(CubicWebTC):

    configcls = PostgresApptestConfiguration

    def test_cwuri_with_ark(self):
        with self.admin_access.repo_cnx() as cnx:
            scheme = cnx.create_entity('ConceptScheme', ark_naa=testutils.naa(cnx))
            cnx.commit()
            cnx.call_service('lcsv.skos.import', scheme_uri=scheme.cwuri,
                             stream=open(self.datapath('lcsv_example_shortened.csv')),
                             delimiter='\t', encoding='utf-8', language_code='es')
            concept1 = cnx.find(
                'Concept', definition="Définition de l'organisation politique de l'organisme").one()
            self.assertTrue(testutils.match_ark_uri(concept1.cwuri),
                            concept1.cwuri)


if __name__ == '__main__':
    import unittest
    unittest.main()



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem_ref common test tools"""

from __future__ import print_function

from os.path import join, dirname, exists
import re
import sys

from doctest import Example

from lxml import etree
from lxml.doctestcompare import LXMLOutputChecker

from logilab.common.decorators import monkeypatch

from cubicweb import NoResultError, devtools
from cubicweb.devtools import testlib

from cubicweb_seda import testutils as seda_testutils
from cubicweb_saem_ref import ark


# make some names them appear in this module namespace to ease test API
assertValidationError = seda_testutils.assertValidationError
assertUnauthorized = seda_testutils.assertUnauthorized
create_data_object = seda_testutils.create_data_object


def create_archive_unit(parent, *args, **kwargs):
    """Create a SEDAArchiveTransfer with an "ark_naa" relation set."""
    if 'ark_naa' not in kwargs:
        cnx = kwargs.get('cnx', getattr(parent, '_cw', None))
        assert cnx is not None
        authority = authority_with_naa(cnx)
        authority.cw_clear_all_caches()
        kwargs['ark_naa'] = authority.ark_naa
    return seda_testutils.create_archive_unit(parent, *args, **kwargs)


def _authority(func):
    """Decorator binding an Organization with an NAA configured in
    kwargs['authority'].
    """
    def wrapper(cnx, *args, **kwargs):
        authority = kwargs.get('authority')
        if authority is None:
            kwargs['authority'] = authority_with_naa(cnx)
        else:
            if isinstance(authority, int):
                authority = cnx.entity_from_eid(authority)
            if not authority.ark_naa:
                with cnx.security_enabled(False, False):
                    authority.cw_set(ark_naa=naa(cnx))
        return func(cnx, *args, **kwargs)
    return wrapper


@_authority
def agent(cnx, name, **kwargs):
    """Return an Agent with specified name."""
    return cnx.create_entity('Agent', name=name, **kwargs)


@_authority
def organization_unit(cnx, name, archival_roles=(), **kwargs):
    """Return an OrganizationUnit with specified name and archival roles."""
    roles_eid = [cnx.find('ArchivalRole', name=role)[0][0] for role in archival_roles]
    return cnx.create_entity('OrganizationUnit', name=name,
                             archival_role=roles_eid, **kwargs)


def authority_record(cnx, name, kind=u'person', **kwargs):
    """Return an AuthorityRecord with specified kind and name."""
    kind_eid = cnx.find('AgentKind', name=kind)[0][0]
    if 'ark_naa' not in kwargs:
        authority = authority_with_naa(cnx)
        authority.cw_clear_all_caches()
        kwargs['ark_naa'] = authority.ark_naa
    record = cnx.create_entity('AuthorityRecord',
                               agent_kind=kind_eid, **kwargs)
    cnx.create_entity('NameEntry', parts=name, form_variant=u'authorized',
                      name_entry_for=record)
    return record


def seda_transfer(cnx, **kwargs):
    """Return a 2.0, 1.0, 0.2 compatible SEDATransfer."""
    transfer = setup_profile(cnx)
    cnx.create_entity('SEDAAccessRule',  # mandatory for seda 1.0
                      seda_access_rule=transfer,
                      seda_seq_access_rule_rule=cnx.create_entity(
                          'SEDASeqAccessRuleRule',
                          reverse_seda_start_date=cnx.create_entity('SEDAStartDate')))
    return transfer


def naa(cnx):
    try:
        return cnx.find('ArkNameAssigningAuthority').one()
    except NoResultError:
        with cnx.security_enabled(False, False):
            return cnx.create_entity('ArkNameAssigningAuthority', who=u'TEST', what=0)


def authority_with_naa(cnx, name=u'Default authority', **kwargs):
    try:
        authority = cnx.find('Organization', name=name).one()
        assert not kwargs, 'extra kwargs may not be specified for a pre-existing organization'
    except NoResultError:
        return cnx.create_entity('Organization', name=name, ark_naa=naa(cnx), **kwargs)
    if not authority.ark_naa:
        with cnx.security_enabled(False, False):
            authority.cw_set(ark_naa=naa(cnx))
    return authority


ARK_URI_RGX = re.compile(
    r'^ark:/0/%s.{%d}%s(/.{%d})?$'
    % (ark.ARK_PREFIX,
       ark.ARK_NAME_LENGTH - len(ark.ARK_PREFIX) - len(ark.ARK_CONTROLCHAR),
       ark.ARK_CONTROLCHAR, ark.ARK_QUALIFIER_LENGTH),
)


ARK_RGX = re.compile(
    r'^0/%s.{%d}%s(/.{%d})?$'
    % (ark.ARK_PREFIX,
       ark.ARK_NAME_LENGTH - len(ark.ARK_PREFIX) - len(ark.ARK_CONTROLCHAR),
       ark.ARK_CONTROLCHAR, ark.ARK_QUALIFIER_LENGTH),
)


ARK_NAME_RGX = re.compile(
    r'^%s.{%d}%s$'
    % (ark.ARK_PREFIX,
       ark.ARK_NAME_LENGTH - len(ark.ARK_PREFIX) - len(ark.ARK_CONTROLCHAR),
       ark.ARK_CONTROLCHAR),
)


def match_ark_uri(text):
    return ARK_URI_RGX.match(text)


def match_ark(text):
    return ARK_RGX.match(text)


def match_ark_name(text):
    return ARK_NAME_RGX.match(text)


def setup_scheme(cnx, title, *labels, **kwargs):
    """Return info new concept scheme"""
    scheme = cnx.create_entity('ConceptScheme', title=title, ark_naa=naa(cnx), **kwargs)
    for label in labels:
        scheme.add_concept(label)
    return scheme


def setup_profile(cnx, **kwargs):
    """Return a minimal SEDA profile."""
    kwargs.setdefault('title', u'Test profile')
    return cnx.create_entity('SEDAArchiveTransfer', ark_naa=naa(cnx), **kwargs)


def concept(cnx, label):
    """Return concept entity with the given preferred label (expected to be unique)."""
    return cnx.execute('Concept X WHERE X preferred_label L, L label %(label)s',
                       {'label': label}).one()


# monkey patch scheme_for_rtype to ensure every scheme is created with a naa set
orig_scheme_for_type = seda_testutils.scheme_for_type


@monkeypatch(seda_testutils)
def scheme_for_type(cnx, rtype, etype, *concept_labels):
    return orig_scheme_for_type(cnx, rtype, etype, *concept_labels, ark_naa=naa(cnx))


class XmlTestMixin(object):
    """Mixin class provinding additional assertion methods for checking XML data."""

    def assertXmlEqual(self, actual, expected):
        """Check that both XML strings represent the same XML tree."""
        checker = LXMLOutputChecker()
        if not checker.check_output(expected, actual, 0):
            message = checker.output_difference(Example("", expected), actual, 0)
            self.fail(message)

    def assertXmlValid(self, xml_data, xsd_filename, debug=False):
        """Validate an XML file (.xml) according to an XML schema (.xsd)."""
        with open(xsd_filename) as xsd:
            xmlschema = etree.XMLSchema(etree.parse(xsd))
        # Pretty-print xml_data to get meaningful line information.
        xml_data = etree.tostring(etree.fromstring(xml_data), pretty_print=True)
        xml_data = etree.fromstring(xml_data)
        if debug and not xmlschema.validate(xml_data):
            print(etree.tostring(xml_data, pretty_print=True))
        xmlschema.assertValid(xml_data)


def startpgcluster(pyfile):
    """Let PostgreSQL cluster be started only once per process and stopped at exit.
    """
    if devtools.DEFAULT_PSQL_SOURCES['system']['db-host'] == 'REPLACEME':
        devtools.startpgcluster(pyfile)
        import atexit
        atexit.register(devtools.stoppgcluster, pyfile)


# speed up tests by using a global configuration ###################################################
_CONFIGS = {}


@monkeypatch(testlib.CubicWebTC, methodname='setUpClass')
@classmethod
def setUpClass(cls):
    try:
        config = _CONFIGS[cls.configcls]
    except KeyError:
        # can't use original implementation, else apphome is not correctly
        # detected
        test_module = sys.modules[cls.__module__]
        test_dir = dirname(test_module.__file__)
        config = cls.configcls('data', join(test_dir, 'data'))
        config.mode = 'test'
        assert exists(config._apphome), config._apphome
        _CONFIGS[cls.configcls] = config
    # force call to init_config which may define a particular setup for the fixture
    cls.init_config(config)
    cls.config = config


class _HCache(dict):
    """Original devtools handler cache prevent caching of several configurations, but that's
    what we're trying to achieve.
    """
    def set(self, config, handler):
        self[config] = handler


devtools.HCACHE = _HCache()



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref entity's classes for exporting agents to eac"""

from cubicweb.predicates import is_instance

from cubes.oaipmh import MetadataFormat
from cubes.oaipmh.entities import RelatedEntityOAISetSpec
from cubicweb_eac import entities as eac

from . import oai


ARECORD_METADATA_FORMATS = {
    'eac': (
        MetadataFormat(
            schema='http://eac.staatsbibliothek-berlin.de/schema/cpf.xsd',
            namespace='urn:isbn:1-931666-33-4'),
        'eac.export',
    ),
}


eac.AuthorityRecord.fetch_attrs += ('ark',)


class AuthorityRecordEACAdapter(eac.AuthorityRecordEACAdapter):

    @property
    def file_name(self):
        """Return a file name for the dump."""
        if self.entity.ark:
            name = self.entity.ark.replace("/", "-")
            return u'EAC_{0}.xml'.format(name)
        return super(AuthorityRecordEACAdapter, self).file_name

    def recordid_element(self, control_elt):
        # export ark in <recordId>
        self.element('recordId', parent=control_elt, text=self.entity.ark.replace("/", "-"))
        record_id = self.entity.record_id
        if record_id:
            # export original <recordId> to <otherRecordId>
            self.element('otherRecordId', parent=control_elt, text=record_id)
        for local_type, value in self.entity.other_record_ids:
            attrs = {}
            if local_type is not None:
                attrs['localType'] = local_type
            self.element('otherRecordId', parent=control_elt, attributes=attrs,
                         text=value)

    def publication_status_element(self, control_elt):
        status = {'draft': 'inProcess', 'published': 'approved'}.get(
            self.entity.cw_adapt_to('IWorkflowable').state, 'inProcess')
        self.element('publicationStatus', parent=control_elt, text=status)

    def agent_element(self, activity, maintenance_event_elt):
        if activity.associated_with:
            agent = activity.associated_with[0].login
        else:
            agent = activity.agent
        if agent:
            self.element('agentType', maintenance_event_elt, text='human')
            self.element('agent', maintenance_event_elt, text=agent)
        else:
            super(AuthorityRecordEACAdapter, self).agent_element(activity, maintenance_event_elt)


class AuthorityRecordOAIPMHRecordAdapter(oai.OAIPMHActiveRecordAdapter):
    """OAI-PMH adapter for AuthorityRecord entity type."""
    __select__ = oai.OAIPMHActiveRecordAdapter.__select__ & is_instance('AuthorityRecord')
    metadata_formats = ARECORD_METADATA_FORMATS.copy()

    @classmethod
    def set_definition(cls):
        specifier = oai.PublicETypeOAISetSpec('AuthorityRecord', cls.identifier_attribute)
        specifier['used_by'] = RelatedEntityOAISetSpec(
            'use_authorityrecord', 'OrganizationUnit', 'ark', role='object',
            description=u'An authority record used by {0}')
        return specifier


class DeletedAuthorityRecordOAIPMHRecordAdapter(oai.OAIPMHDeletedRecordAdapter):
    """OAI-PMH adapter for deleted AuthorityRecord."""
    __select__ = oai.OAIPMHDeletedRecordAdapter.__select__ & is_instance('AuthorityRecord')
    metadata_formats = ARECORD_METADATA_FORMATS.copy()


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (AuthorityRecordEACAdapter,))
    vreg.register_and_replace(AuthorityRecordEACAdapter, eac.AuthorityRecordEACAdapter)



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as publishged by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref test for OAI-PMH export"""

from collections import namedtuple
from functools import wraps
import time

from dateutil.parser import parse as parse_date
from lxml import etree
import pytz

from cubicweb.devtools import PostgresApptestConfiguration
from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.pyramid.test import PyramidCWTest

from cubes.oaipmh.views import OAIError, OAIRequest

from cubicweb_saem_ref import permanent_url

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


EntityInfo = namedtuple('EntityInfo', ['ark', 'url', 'eid'])


def publish_entity(entity):
    """Fire "publish" transition for `entity`."""
    entity.cw_adapt_to('IWorkflowable').fire_transition('publish')
    entity._cw.commit()


def deprecate_entity(entity):
    """Fire "deprecate" transition for `entity`."""
    entity.cw_adapt_to('IWorkflowable').fire_transition('deprecate')
    entity._cw.commit()


class OAITestMixin(object):

    def oai_component(self, cnx):
        """Return the "oai" component"""
        return self.vreg['components'].select('oai', cnx)


class OAIComponentTC(CubicWebTC, OAITestMixin):

    configcls = PostgresApptestConfiguration

    def test_registered(self):
        with self.admin_access.repo_cnx() as cnx:
            oai = self.oai_component(cnx)
            setspecs = oai.__setspecs__
            self.assertCountEqual(setspecs.keys(),
                                  ['agent', 'organizationunit', 'organization',
                                   'authorityrecord',
                                   'conceptscheme', 'profile', 'concept'])
            self.assertCountEqual(setspecs['organizationunit'].keys(), ['role'])
            self.assertCountEqual(setspecs['profile'].keys(),
                                  ['transferring_agent'])
            self.assertEqual(setspecs['conceptscheme'].keys(), [])
            self.assertEqual(setspecs['authorityrecord'].keys(), ['used_by'])

    def test_setspecs(self):
        with self.admin_access.repo_cnx() as cnx:
            arch = testutils.organization_unit(
                cnx, u'arch', archival_roles=[u'archival'])
            ark = testutils.organization_unit(
                cnx, u'dep', archival_roles=[u'deposit']).ark
            scheme = testutils.setup_scheme(cnx, u'test', u'lab')
            cnx.commit()
            publish_entity(scheme)
            expected = (
                'agent',
                'authorityrecord',
                'authorityrecord:used_by:%s' % arch.ark,
                'authorityrecord:used_by:%s' % ark,
                'organization',
                'organizationunit',
                'organizationunit:role:producer',
                'organizationunit:role:deposit',
                'organizationunit:role:archival',
                'organizationunit:role:control',
                'organizationunit:role:enquirer',
                'organizationunit:role:seda-actor',
                'conceptscheme',
                'profile', 'profile:transferring_agent:%s' % ark,
                'concept', 'concept:in_scheme:%s' % scheme.ark
            )
            oai = self.oai_component(cnx)
            setspecs = [x[0] for x in oai.setspecs()]
            self.assertCountEqual(setspecs, expected)


def setup_organization_units(cnx):
    """Create two agents (alice and bobby) in published state and return their
    (ark, url, eid) attributes.
    """
    alice = testutils.organization_unit(cnx, u'alice',
                                        archival_roles=[u'producer'])
    cnx.commit()
    bobby = testutils.organization_unit(cnx, u'bobby',
                                        archival_roles=[u'enquirer'])
    cnx.commit()

    return (
        EntityInfo(alice.ark, permanent_url(alice), alice.eid),
        EntityInfo(bobby.ark, permanent_url(bobby), bobby.eid),
    )


class OAIRequestTC(CubicWebTC):

    configcls = PostgresApptestConfiguration

    def test_rset(self):
        """Test `rset` function for exceptions."""
        with self.admin_access.repo_cnx() as cnx:
            return setup_organization_units(cnx)
        with self.admin_access.web_request() as req:
            for setspec, msg in [
                ('organizationunit:zen', 'invalid set specifier organizationunit:zen'),
                ('organizationunit:role:control', (
                    'The combination of the values of the from, until, and '
                    'set arguments results in an empty list.')),
                ('conceptscheme:haha', 'invalid set specifier conceptscheme:haha'),
                ('scregneugneu', 'invalid set specifier scregneugneu'),
            ]:
                with self.subTest(setspec=setspec):
                    oairq = OAIRequest('https://localhost:80', setspec=setspec)
                    with self.assertRaises(OAIError) as cm:
                        oairq.rset_from_setspec(req)
                    self.assertEqual(cm.exception.errors, {'noRecordsMatch': msg})


def no_validate_xml(method):
    """Disable XML schema validation, often because the underlying metadata
    part of the response (RDF, XSD) is not validable (or we don't know how to
    do it).
    """
    @wraps(method)
    def wrapper(self):
        self._validate_xml = False
        self._debug_xml = True
        return method(self)
    return wrapper


class OAIPMHViewsTC(PyramidCWTest, OAITestMixin, testutils.XmlTestMixin):

    configcls = PostgresApptestConfiguration
    settings = {
        # to get clean traceback in tests (instead of in an html error page)
        'cubicweb.bwcompat': False,
        # avoid noise in test output (UserWarning: !! WARNING WARNING !! you
        # should stop this instance)
        'cubicweb.session.secret': 'x',
        'cubicweb.auth.authtkt.session.secret': 'x',
        'cubicweb.auth.authtkt.persistent.secret': 'x',
    }

    _validate_xml = True
    _debug_xml = False

    def oai_request(self, req, **formparams):
        response = self.webapp.get('/oai', formparams)
        self.assertEqual(response.headers['Content-Type'], 'text/xml; charset=UTF-8')
        if self._validate_xml:
            self.assertXmlValid(response.body, self.datapath('OAI-PMH.xsd'),
                                debug=self._debug_xml)
        return response.body

    def _setup_organization_units(self, **kwargs):
        with self.admin_access.repo_cnx() as cnx:
            return setup_organization_units(cnx, **kwargs)

    def test_xml_attributes_and_namespaces(self):
        """Check XML attributes and namespace declaration of the response."""
        with self.admin_access.web_request() as req:
            # simple request, generating an error but enough to get a properly
            # formatter response.
            result = self.oai_request(req)
            xml = etree.fromstring(result)
            nsmap = {None: 'http://www.openarchives.org/OAI/2.0/',
                     'xsi': 'http://www.w3.org/2001/XMLSchema-instance'}
            self.assertEqual(xml.nsmap, nsmap)
            attrib = {
                '{http://www.w3.org/2001/XMLSchema-instance}schemaLocation': ' '.join(
                    ['http://www.openarchives.org/OAI/2.0/',
                     'http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd']),
            }
            self.assertEqual(xml.attrib, attrib)

    def _setup_schemes(self):
        with self.admin_access.repo_cnx() as cnx:
            pscheme = testutils.setup_scheme(cnx, u'public',
                                             u'public label', u'other public label')
            # draft ConceptScheme
            dscheme = testutils.setup_scheme(cnx, u'draft',
                                             u'draft label', u'other draft label')
            cnx.commit()
            publish_entity(pscheme)
            return (
                EntityInfo(pscheme.ark, pscheme.absolute_url(), pscheme.eid),
                EntityInfo(dscheme.ark, dscheme.absolute_url(), dscheme.eid),
            )

    def test_noverb(self):
        with self.admin_access.web_request() as req:
            result = self.oai_request(req)
        self.assertIn('<request>https://localhost:80/oai</request>',
                      result)
        self.assertIn('<error code="badVerb">no verb specified</error',
                      result)

    def test_badverb(self):
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='coucou')
        self.assertIn('<request>https://localhost:80/oai</request>',
                      result)
        self.assertIn('<error code="badVerb">illegal verb "coucou"</error>',
                      result)

    # some of our setspecs (those with an ARK inside are not valid)
    @no_validate_xml
    def test_listsets(self):
        pscheme, dscheme = self._setup_schemes()
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='ListSets')
        self.assertIn('<request verb="ListSets">https://localhost:80/oai</request>',
                      result)
        for spec in ('agent', 'organizationunit', 'conceptscheme', 'profile',
                     'concept'):
            self.assertIn('<setSpec>{0}</setSpec>'.format(spec),
                          result)
        self.assertIn(('<setSpec>organizationunit:role:deposit</setSpec>'
                       '<setName>An organization unit with deposit archival role</setName>'),
                      result)
        self.assertIn('<setSpec>concept:in_scheme:{0}</setSpec>'.format(pscheme.ark),
                      result)
        self.assertNotIn(dscheme.ark, result)

    def _setup_profile(self, **kwargs):
        """Create a "published" SEDAArchiveTransfer and return it."""
        with self.admin_access.repo_cnx() as cnx:
            profile = testutils.setup_profile(cnx, **kwargs)
            cnx.commit()
            publish_entity(profile)
            profile.ark  # prefetch attribute for later usage
            return profile

    def test_listidentifiers(self):
        alice, bobby = [x.ark for x in self._setup_organization_units()]
        profile = self._setup_profile()
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='ListIdentifiers',
                                      set='organizationunit',
                                      metadataPrefix='rdf')
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice), result)
            result = self.oai_request(req, verb='ListIdentifiers',
                                      set='organizationunit:role:producer',
                                      metadataPrefix='rdf')
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice),
                          result)
            self.assertNotIn('<identifier>ark:/{0}</identifier>'.format(bobby),
                             result)
            result = self.oai_request(req, verb='ListIdentifiers',
                                      set='organizationunit:role:producer',
                                      metadataPrefix='rdf')
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice), result)
            self.assertNotIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
            result = self.oai_request(req, verb='ListIdentifiers',
                                      set='conceptscheme',
                                      metadataPrefix='rdf')
            result = self.oai_request(req, verb='ListIdentifiers',
                                      set='profile', metadataPrefix='seda02xsd')
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile.ark), result)

    def test_listidentifiers_deleted(self):
        """Check <header> element (status attribute, datestamp) for deleted
        records in ListIdentifiers response.
        """
        alice, bobby = self._setup_organization_units()
        with self.admin_access.repo_cnx() as cnx:
            entity = cnx.entity_from_eid(bobby.eid)
            # ensure transition date is greater by at least 1s
            time.sleep(1)
            date_before = entity.modification_date.replace(tzinfo=pytz.utc)
            deprecate_entity(entity)

        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='ListIdentifiers',
                                      set='organizationunit',
                                      metadataPrefix='rdf')
            self.assertIn(
                '<identifier>ark:/{0}</identifier>'.format(bobby.ark), result)
            self.assertIn('<header status="deleted">', result)
            root = etree.fromstring(result)
            ns = root.nsmap[None]
            # only one <header status="..."> element
            header, = root.findall('.//{%s}header[@status]' % ns)
            datestamp = header.find('{%s}datestamp' % ns).text
            date = parse_date(datestamp)
            self.assertGreater(date, date_before.replace(microsecond=0))

    @no_validate_xml
    def test_listrecords(self):
        alice, bobby = self._setup_organization_units()
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='ListRecords', set='organizationunit',
                                      metadataPrefix='rdf')
            self.assertIn(
                '<rdf:Description rdf:about="{0}">'.format(bobby.url), result)
            self.assertIn(
                '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
            result = self.oai_request(
                req, verb='ListRecords', set='organizationunit:role:producer',
                metadataPrefix='rdf')
            self.assertIn(
                '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
            self.assertNotIn(bobby.url, result)

    @no_validate_xml
    def test_listrecords_profile(self):
        self._setup_profile()
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='ListRecords', set='profile',
                                      metadataPrefix='seda02xsd')
        # Just ensure a SEDA profile (xsd) is present in response.
        self.assertIn('<metadata><xsd:schema', result)
        self.assertIn('fr:gouv:ae:archive:draft:standard_echange_v0.2', result)

    @no_validate_xml
    def test_listidentifiers_profile_setspec(self):
        """Check setspec restriction for SEDAArchiveTransfer."""
        profile = self._setup_profile()
        with self.admin_access.cnx() as cnx:
            tagent_eid = testutils.organization_unit(
                cnx, u'blah', archival_roles=[u'deposit']).eid
            cnx.commit()
        profile2 = self._setup_profile()
        with self.admin_access.cnx() as cnx:
            cnx.entity_from_eid(tagent_eid).cw_set(use_profile=profile2.eid)
            cnx.commit()
        with self.admin_access.web_request() as req:
            transferring_agent_ark = req.entity_from_eid(tagent_eid).ark
            result = self.oai_request(
                req, verb='ListIdentifiers',
                set='profile:transferring_agent:' + transferring_agent_ark,
                metadataPrefix='seda02xsd')
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile2.ark), result)
            self.assertNotIn(profile.ark, result)
        # Check that without a setspec, we get all profiles.
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='ListIdentifiers',
                                      set='profile', metadataPrefix='seda02xsd')
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile.ark), result)
            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile2.ark), result)

    @no_validate_xml
    def test_getrecord_profile(self):
        profile = self._setup_profile()
        with self.admin_access.web_request() as req:
            with self.subTest(md_prefix='seda02xsd'):
                result = self.oai_request(req, verb='GetRecord',
                                          identifier='ark:/' + profile.ark,
                                          metadataPrefix='seda02xsd')
                self.assertIn('<metadata><xsd:schema', result)
                self.assertIn('http://www.w3.org/2001/XMLSchema', result)
            for md_prefix in ['seda02rng', 'seda1rng', 'seda2rng']:
                with self.subTest(md_prefix=md_prefix):
                    result = self.oai_request(req, verb='GetRecord',
                                              identifier='ark:/' + profile.ark,
                                              metadataPrefix=md_prefix)
                    self.assertIn('<metadata><rng:grammar', result)
                    self.assertIn('http://www.w3.org/2001/XMLSchema', result)

    @no_validate_xml
    def test_getrecord_agent(self):
        self._debug_xml = True
        with self.admin_access.repo_cnx() as cnx:
            agent = testutils.agent(cnx, u'toto')
            cnx.commit()
            ark = agent.ark
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='GetRecord',
                                      identifier='ark:/' + ark,
                                      metadataPrefix='rdf')
            self.assertIn('name>toto</', result)
            self.assertIn('<rdf:type rdf:resource="http://xmlns.com/foaf/0.1/Person"/>',
                          result)

    @no_validate_xml
    def test_getrecord_organization(self):
        with self.admin_access.repo_cnx() as cnx:
            org = testutils.authority_with_naa(cnx)
            cnx.commit()
            ark = org.ark
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='GetRecord',
                                      identifier='ark:/' + ark,
                                      metadataPrefix='rdf')
            self.assertIn('title>Default authority</', result)
            self.assertIn('<rdf:type rdf:resource="http://www.w3.org/ns/org#Organization"/>',
                          result)

    @no_validate_xml
    def test_getrecord_organizationunit(self):
        self._debug_xml = True
        with self.admin_access.repo_cnx() as cnx:
            orgu = testutils.organization_unit(cnx, u'toto')
            cnx.commit()
            ark = orgu.ark
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='GetRecord',
                                      identifier='ark:/' + ark,
                                      metadataPrefix='rdf')
            self.assertIn('title>toto</', result)
            self.assertIn('<rdf:type rdf:resource="http://www.w3.org/ns/org#OrganizationUnit"/>',
                          result)

    @no_validate_xml
    def test_getrecord_authority_record(self):
        with self.admin_access.repo_cnx() as cnx:
            arecord = testutils.authority_record(cnx, u'M. Person')
            cnx.commit()
            arecord.cw_adapt_to('IWorkflowable').fire_transition('publish')
            cnx.commit()
            arecord_ark = arecord.ark
        with self.admin_access.web_request() as req:
            result = self.oai_request(req, verb='GetRecord',
                                      identifier='ark:/' + arecord_ark,
                                      metadataPrefix='eac')
            self.assertIn('<metadata><eac-cpf', result)
            self.assertIn('urn:isbn:1-931666-33-4', result)

    @no_validate_xml
    def test_authorityrecord_from_organizationunit(self):
        with self.admin_access.cnx() as cnx:
            org = testutils.authority_with_naa(cnx)
            ou_ark = testutils.organization_unit(
                cnx, u'test', archival_roles=('archival', ),
                reverse_archival_unit=org, authority=org).ark
            self.create_user(cnx, u'bob', authority=org)
            cnx.commit()
        with self.new_access(u'bob').cnx() as cnx:
            arecord = testutils.authority_record(cnx, u'test')
            cnx.commit()
            assert arecord.reverse_use_authorityrecord
            arecord.cw_adapt_to('IWorkflowable').fire_transition('publish')
            cnx.commit()
            ark = arecord.ark
        with self.admin_access.web_request() as req:
            result = self.oai_request(
                req, verb='ListIdentifiers',
                set='authorityrecord:used_by:{}'.format(ou_ark),
                metadataPrefix='eac')
            self.assertIn('<identifier>ark:/' + ark, result)


if __name__ == '__main__':
    import unittest
    unittest.main()



# coding: utf-8
# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem_ref unit tests for views"""

import json

from logilab.common import flatten

from cubicweb.devtools import PostgresApptestConfiguration
from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_saem_ref.views.autocompletesearch import words_matching_term

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


def sql(cnx, string, param=None):
    return cnx.system_sql(string, param).fetchall()


class AutocompleteSearchTC(CubicWebTC):
    configcls = PostgresApptestConfiguration

    def test_syncro_concepts(self):
        with self.admin_access.client_cnx() as cnx:
            self.assertFalse(sql(cnx, 'SELECT * from words WHERE word=%(w)s;',
                                 {'w': u'déclaration'}))
            scheme = cnx.create_entity('ConceptScheme', title=u'my thesaurus',
                                       ark_naa=testutils.naa(cnx))
            concept = scheme.add_concept(label=u'Déclaration universelle', language_code=u'fr',
                                         definition=u"Déclaration des droits de l'homme")
            scheme.add_concept(label=u'The Universal Declaration', language_code=u'fr',
                               definition=u'Declaration of human rights and citizen')
            other_scheme = cnx.create_entity('ConceptScheme', ark_naa=testutils.naa(cnx))
            other_scheme.add_concept(u'25 ans - secret statistique')
            other_scheme.add_concept(
                u'100 ans- personnes mineures',
                definition=(u'Documents évoquant des personnes mineures : '
                            u'statistiques, enquêtes de la police judiciaire, ...')
            )
            yet_another_scheme = cnx.create_entity('ConceptScheme', ark_naa=testutils.naa(cnx))
            yet_another_scheme.add_concept(
                u'fmt/210', definition=u'Statistica Report File Statistica Report File')
            cnx.commit()

            all_words = flatten(sql(cnx, 'SELECT word FROM words'))
            for word in (u'droits', u'd\xe9claration', u'citizen', u'rights'):
                self.assertIn(word, all_words)

        with self.admin_access.web_request() as req:
            self.assertEqual(words_matching_term(req, u'déclaration'),
                             [(u'd\xe9claration', u'Concept'),
                              (u'declaration', u'Concept')])
            self.assertEqual(words_matching_term(req, u'statisti'),
                             [(u'statistica', u'Concept'),
                              (u'statistique', u'Concept'),
                              (u'statistiques', u'Concept')])
            self.assertEqual(words_matching_term(req, 'rspons'), [])
            concept = req.cnx.entity_from_eid(concept.eid)
            concept.preferred_label[0].cw_set(label=u'Loi renseignement')
            concept.cw_set(definition=u'''L'Assemblée et le Sénat se sont mis d'accord, mardi 16
            juin, sur une version définitive de la loi renseignement. Un texte voté dans une sorte
            d'indifférence générale et qui dote la France d'une des lois les plus intrusives
            d’Europe.''')
            req.cnx.commit()
            self.assertEqual(words_matching_term(req, 'déclaration'),
                             [(u'declaration', u'Concept')])

    def _add_some_content(self):
        with self.admin_access.client_cnx() as cnx:
            testutils.authority_record(cnx, u'Secret record')
            scheme = cnx.create_entity('ConceptScheme', ark_naa=testutils.naa(cnx))
            scheme.add_concept(u'Top secret')
            cnx.commit()

    def _search(self, **formdata):
        formdata.setdefault('facetrql', '')
        with self.admin_access.web_request(**formdata) as req:
            view = self.vreg['views'].select('search-autocomplete', req)
            return json.loads(view.render())

    def test_empty_search(self):
        words = self._search(term='secretos', all='secretos')
        self.assertEqual(words, [])

    def test_search(self):
        self._add_some_content()
        words = self._search(term='secret', all='secret')
        self.assertEqual(words, [{'label': 'secret : AuthorityRecord, Concept', 'value': 'secret'}])
        words = self._search(term='secret', all='record secret')
        self.assertEqual(words, [{'label': 'secret : AuthorityRecord', 'value': 'secret'}])

    def test_facetted_search(self):
        self._add_some_content()
        words = self._search(term='secret', all='secret',
                             facetrql='Any X WHERE X is AuthorityRecord')
        self.assertEqual(words, [{'label': 'secret : AuthorityRecord', 'value': 'secret'}])


if __name__ == '__main__':
    import unittest
    unittest.main()



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-prov views/forms/actions/components for web ui"""

from logilab.mtconverter import xml_escape

from cubicweb.predicates import relation_possible
from cubicweb.web.views import tableview

from cubicweb_prov import views as prov

from . import SubviewsTabView


class LifeCycleTabView(SubviewsTabView):
    """Life-cycle tab for concept and concept scheme."""
    __regid__ = 'saem.lifecycle_tab'
    __select__ = relation_possible('generated', role='object')
    subvids = ('prov.activity-generated',
               'prov.activity-associated-with')


class ActivityGeneratedView(prov.ActivityGeneratedView):
    rql = ('Any E,T,DE,DEF,D,A,U ORDERBY D DESC '
           'WHERE E generated X, E agent A, E associated_with U?, E type T, E start D,'
           '      E description DE, E description_format DEF, X eid %(x)s')


class ActivityAgentColRenderer(tableview.MainEntityColRenderer):

    def render_header(self, w):
        w(self._cw.__('agent'))

    def render_entity(self, w, entity):
        if entity.associated_with:
            w(entity.associated_with[0].view('incontext'))
        elif entity.agent:
            w(xml_escape(entity.agent))
        else:
            w(self.empty_cell_content)

    def entity_sortvalue(self, entity):
        if entity.associated_with:
            return entity.associated_with[0].name()
        elif entity.agent:
            return entity.agent
        else:
            return None


class ActivityGeneratedTable(prov.ActivityGeneratedTable):
    column_renderers = {
        'agent': ActivityAgentColRenderer(),
    }


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__,
                      [ActivityGeneratedView, ActivityGeneratedTable])
    vreg.register_and_replace(ActivityGeneratedView, prov.ActivityGeneratedView)
    vreg.register_and_replace(ActivityGeneratedTable, prov.ActivityGeneratedTable)



for ertype in ('Activity', 'associated_with', 'generated', 'used'):
    sync_schema_props_perms(ertype)

add_relation_definition('Activity', 'generated', 'SEDAArchiveTransfer')
add_relation_definition('Activity', 'used', 'SEDAArchiveTransfer')



# coding: utf-8
# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem_ref unit tests for dataimport"""

from os.path import basename

from cubicweb.dataimport.importer import SimpleImportLog
from cubicweb.devtools import PostgresApptestConfiguration
from cubicweb.devtools.testlib import CubicWebTC

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class EACDataImportTC(CubicWebTC):

    configcls = PostgresApptestConfiguration

    def setup_database(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_user(cnx, u'bob', ('users', ),
                             authority=testutils.authority_with_naa(cnx))
            cnx.commit()

    def test_imported_activities(self):
        fpath = self.datapath('EAC', 'FRAD033_EAC_dataimport.xml')
        with self.new_access(u'bob').cnx() as cnx:
            import_log = SimpleImportLog(basename(fpath))
            cnx.call_service(
                'eac.import', stream=fpath, import_log=import_log,
                raise_on_error=True)
            rset = cnx.find('AuthorityRecord', isni=u'22330001300016')
            record = rset.one()
            rset = cnx.execute('Any A WHERE A generated X, X eid %(x)s', {'x': record.eid})
            # Two activities imported from EAC-CPF file, one created by our hook.
            self.assertEqual(len(rset), 3)


if __name__ == '__main__':
    import unittest
    unittest.main()



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref automatic tests"""

from functools import partial
import random
from unittest import TestCase
from logilab.common import attrdict

from cubicweb.devtools.fill import ValueGenerator
from cubicweb.devtools import (
    PostgresApptestConfiguration,
    testlib,
)
from cubicweb.schema import SCHEMA_TYPES

from cubicweb_saem_ref import cwuri_url

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class CWURI_URLTC(TestCase):

    def test(self):
        class entity(attrdict):
            @property
            def _cw(self):
                return self

            def build_url(self, path):
                return 'http://built/' + path

        self.assertEqual(cwuri_url(entity({'cwuri': 'whatever'})),
                         'whatever')
        self.assertEqual(cwuri_url(entity({'cwuri': 'ark:/123'})),
                         'http://built/ark:/123')
        self.assertEqual(cwuri_url(entity({'cwuri': 'http://domain.org/ark:/123'})),
                         'http://domain.org/ark:/123')


class ArkURLTC(testlib.CubicWebTC):

    configcls = PostgresApptestConfiguration

    def setup_database(self):
        with self.admin_access.cnx() as cnx:
            testutils.authority_with_naa(cnx)
            cnx.commit()

    def test_absolute_url_use_ark(self):
        ark_etypes = [eschema.type for eschema in self.schema['ark'].subjects()]
        create_entity_for = {
            'Agent': partial(testutils.agent, name=u'bob'),
            'AuthorityRecord': partial(testutils.authority_record, name=u'rec'),
            'Concept': partial(testutils.concept, label=u'l'),
            'ConceptScheme': partial(testutils.setup_scheme, title=u't'),
            'Organization': partial(testutils.authority_with_naa),
            'OrganizationUnit': partial(testutils.organization_unit, name=u'ou'),
            'SEDAArchiveTransfer': testutils.setup_profile,
            'SEDAArchiveUnit': lambda cnx: testutils.create_archive_unit(
                None, cnx=cnx, ark_naa=testutils.naa(cnx))[0],
        }
        baseurl = self.vreg.config['base-url']
        with self.admin_access.cnx() as cnx:
            testutils.setup_scheme(cnx, u'example', u'l')
            cnx.commit()
            for etype in ark_etypes[:]:
                # with self.subTest(etype=etype):
                entity = create_entity_for[etype](cnx)
                assert entity.ark
                url = entity.absolute_url()
                assert url.startswith(baseurl)
                path = url[len(baseurl):]
                self.assertEqual(path, u'ark:/' + entity.ark)
                ark_etypes.remove(etype)
        if ark_etypes:
            self.fail('entity types not checked {}'.format(ark_etypes))


class MyValueGenerator(ValueGenerator):

    def generate_Any_ark(self, entity, index):
        return '/'.join([
            ''.join(random.sample('0123456789', 5)),
            ''.join(random.sample('qwertyuvckj87932141', 10)),
        ]).decode('ascii')


class AutomaticWebTest(testlib.AutomaticWebTest):
    configcls = PostgresApptestConfiguration
    test_ten_each_config = None  # deactivate attempt to auto populate
    skip_views = set([
        'calendar',
        'csvexport',
        'ecsvexport',
        'ejsonexport',
        'filetree',
        'hcal',
        'jsonexport',
        'n3rdf',
        'oneweekcal',
        'onemonthcal',
        'owlabox',
        'rsetxml',
        'rss',
        'security',
        'treeview',
        'xbel',
        'skos.source-sync',
    ])

    @property
    def no_auto_populate(self):
        seda_types_to_skip = [eschema.type for eschema in self.schema.entities()
                              if eschema.type.startswith('SEDA')
                              and eschema != 'SEDAArchiveTransfer']
        eac_types_to_skip = ['EACResourceRelation',
                             'HierarchicalRelation',
                             'ChronologicalRelation',
                             'AssociationRelation']
        cw_types_to_skip = [etype for etype in SCHEMA_TYPES
                            if etype not in ('CWEType', 'CWRType')]
        return seda_types_to_skip + eac_types_to_skip + cw_types_to_skip

    @property
    def ignored_relations(self):
        seda_types_to_skip = [rschema.type for rschema in self.schema.relations()
                              if rschema.type.startswith('seda_')
                              and rschema not in ('seda_archive_unit',
                                                  'seda_binary_data_object')]
        ext_types_to_skip = [rschema.type for rschema in self.schema.relations()
                             if 'ExternalUri' in rschema.objects()
                             or 'ExternalUril' in rschema.subjects()]

        return set(seda_types_to_skip + ext_types_to_skip + ['use_email'])

    def post_populate(self, cnx):
        unit, alt, alt_seq = testutils.create_archive_unit(
            cnx.find('SEDAArchiveTransfer').one(), ark=u'0/dummy')
        testutils.create_data_object(alt_seq)

        cnx.execute('SET L kind "preferred" WHERE NOT EXISTS (L label_of X, L kind "preferred")')

    def to_test_etypes(self):
        '''only test views for entities of the returned types'''
        no_auto_populate = self.no_auto_populate
        # skip email address as well because we drop its alias attribute which makes standard views
        # crashing, while it's only a second class citizen here so it's not actually a problem
        etypes = [eschema.type for eschema in self.schema.entities()
                  if not (eschema.final
                          or eschema.type in no_auto_populate
                          or eschema == 'EmailAddress')]
        return etypes

    def list_views_for(self, rset):
        for view in super(AutomaticWebTest, self).list_views_for(rset):
            if view.__regid__ in self.skip_views:
                continue
            yield view

    # XXX needed until 3.25 is released
    def list_boxes_for(self, rset):
        """returns the list of boxes that can be applied on `rset`"""
        req = rset.req
        for box in self.vreg['ctxcomponents'].possible_objects(req, rset=rset,
                                                               view=None):
            yield box


if __name__ == '__main__':
    import unittest
    unittest.main()



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref views for import/export of AuthorityRecord from/to EAC"""

from six import text_type

from cubicweb import tags
from cubicweb.predicates import specified_etype_implements
from cubicweb.web import formfields as ff
from cubicweb.web.views import editforms

from cubicweb_eac import views as eac

from .. import user_has_naa, user_has_authority


def naa_form_vocabulary(form, field):
    """Field vocabulary function returning the list of available authorities"""
    rset = form._cw.execute('Any XN, X ORDERBY XN WHERE X is ArkNameAssigningAuthority, X who XN')
    return [(name, text_type(eid)) for name, eid in rset]


class EACImportForm(eac.EACImportForm):
    """EAC-CPF import controller - overriden to add an NAA file, necessary to the service."""
    naafield = ff.StringField(name='naa', required=True,
                              choices=naa_form_vocabulary, sort=False)


class EACImportView(eac.EACImportView):
    """EAC-CPF import controller - overriden to provide NAA information to the service."""

    def service_kwargs(self, posted):
        """Subclass access point to provide extra arguments to the service (e.g. saem_ref cube).
        """
        return {'naa': self._cw.cnx.entity_from_eid(posted['naa'])}


class EACImportViewNoNaa(eac.EACImportView):
    __select__ = eac.EACImportView.__select__ & ~user_has_naa()

    def call(self):
        self.w(tags.h1(self._cw.__('Importing an AuthorityRecord from a EAC-CPF file')))
        if not self._cw.user.authority:
            msg = self._cw._("You must be in an organization to access this functionnality.")
            msg += u" <a href='{}'>{}</a>".format(self._cw.user.absolute_url(vid='edition'),
                                                  self._cw._('Edit your settings'))
        else:
            msg = self._cw._("Your organization must have an NAA configured to "
                             "access this functionnality.")
            authority = self._cw.user.authority[0]
            if authority.cw_has_perm('update'):
                msg += u" <a href='{}'>{}</a>".format(authority.absolute_url(vid='edition'),
                                                      self._cw._('Edit authority settings'))
        self.w(tags.div(msg))


class EACCreationFormViewNoNaa(editforms.CreationFormView):
    __select__ = (editforms.CreationFormView.__select__
                  & specified_etype_implements('AuthorityRecord')
                  & ~user_has_authority())

    def render_form(self, entity):
        self.form_title(entity)
        msg = self._cw._("You must be in an organization to access this functionnality.")
        msg += u" <a href='{}'>{}</a>".format(self._cw.user.absolute_url(vid='edition'),
                                              self._cw._('Edit your settings'))
        self.w(tags.div(msg))


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (EACImportForm, EACImportView))
    vreg.register_and_replace(EACImportForm, eac.EACImportForm)
    vreg.register_and_replace(EACImportView, eac.EACImportView)
